@@ -46,6 +46,7 @@ typedef jack_default_audio_sample_t sample_t;
 
 static const size_t rb_n_samples = 53000;       /* maximum number of samples to hold in the ring buffer */
 static uint32_t encoder_packet_magic_number = 'I' << 24 | 'D' << 16 | 'J' << 8 | 'C';
+static const float fade_floor = 0.0003f;
 
 int encoder_init_lame(struct threads_info *ti, struct universal_vars *uv, void *param)
     {
@@ -252,17 +253,6 @@ static long encoder_resampler_get_data(void *cb_data, float **data)
     return (long)n_samples;
     }
 
-static void encoder_apply_pregain(struct encoder_ip_data *id, float gain)
-    {
-    if (gain != 1.0f)
-        for (int i = 0; i < id->channels; ++i)
-            {
-            float *bp = id->buffer[i];
-            for (size_t s = id->qty_samples; s; --s)
-                *bp++ *= gain;
-            }
-    }
-
 struct encoder_ip_data *encoder_get_input_data(struct encoder *encoder, size_t min_samples_needed, size_t max_samples, float **caller_supplied_buffer)
     {
     struct encoder_ip_data *id;
@@ -330,7 +320,27 @@ struct encoder_ip_data *encoder_get_input_data(struct encoder *encoder, size_t m
             goto no_data;
         }
 
-    encoder_apply_pregain(id, encoder->pregain);
+    pthread_mutex_lock(&encoder->fade_mutex);
+    if (encoder->pregain != 1.0f || encoder->fadescale != 1.0f)
+        {
+        float pgain = encoder->pregain;
+        float fgain = 1.0f, fscale = encoder->fadescale; 
+
+        for (int i = 0; i < id->channels; ++i)
+            {
+            float *bp = id->buffer[i];
+            fgain = encoder->fadegain;
+            for (size_t s = id->qty_samples; s; --s)
+                *bp++ *= pgain * (fgain *= fscale); 
+            }
+
+        if (fgain < fade_floor)
+            encoder->fadegain = encoder->fadescale = 1.0f;
+        else
+            encoder->fadegain = fgain;
+        }
+    pthread_mutex_unlock(&encoder->fade_mutex);
+
     return id;
 
     no_data:
@@ -568,7 +578,7 @@ int encoder_start(struct threads_info *ti, struct universal_vars *uv, void *othe
                             break;
                         case ENCODER_CODEC_AAC:
                         case ENCODER_CODEC_AACPLUSV2:
-#if defined(HAVE_AVCODEC) && defined(HAVE_AVFORMAT) && defined(HAVE_AVUTIL)
+#if defined(HAVE_AVCODEC) && defined(HAVE_AVFORMAT)
                             encoder_init = live_avcodec_encoder_init;
 #endif
                             break;
@@ -627,6 +637,7 @@ int encoder_start(struct threads_info *ti, struct universal_vars *uv, void *othe
     self->resample_f = !(self->samplerate == self->target_samplerate);
     self->sr_conv_ratio = (double)self->target_samplerate / (double)self->samplerate;
     self->pregain = atof(ev->pregain);
+    self->fadegain = self->fadescale = 1.0f;
     if (ev->bitrate)
         self->bitrate = atoi(ev->bitrate);
     self->n_channels = strcmp(ev->mode, "mono") ? 2 : 1;
@@ -698,6 +709,18 @@ int encoder_update(struct threads_info *ti, struct universal_vars *uv, void *oth
     encoder_unlink(self);
     return encoder_start(ti, uv, other);
     }
+
+int encoder_initiate_fade(struct threads_info *ti, struct universal_vars *uv, void *other)
+    {
+    struct encoder *self = ti->encoder[uv->tab];
+    
+    pthread_mutex_lock(&self->fade_mutex);
+    if (self->fadescale == 1.0f)
+        self->fadescale = powf(fade_floor, 1.f / (6.f * self->target_samplerate));
+    pthread_mutex_unlock(&self->fade_mutex);
+    
+    return SUCCEEDED;
+    }
  
 int encoder_new_song_metadata(struct threads_info *ti, struct universal_vars *uv, void *other)
     {
@@ -795,6 +818,7 @@ struct encoder *encoder_init(struct threads_info *ti, int numeric_id)
     pthread_mutex_init(&self->mutex, NULL);
     pthread_mutex_init(&self->metadata_mutex, NULL);
     pthread_mutex_init(&self->flush_mutex, NULL);
+    pthread_mutex_init(&self->fade_mutex, NULL);
     if (pthread_create(&self->thread_h, NULL, encoder_main, self))
         {
         fprintf(stderr, "encoder_init: pthread_create call failed\n");
@@ -811,6 +835,7 @@ void encoder_destroy(struct encoder *self)
     pthread_mutex_destroy(&self->mutex);
     pthread_mutex_destroy(&self->metadata_mutex);
     pthread_mutex_destroy(&self->flush_mutex);
+    pthread_mutex_destroy(&self->fade_mutex);
     if (self->rs_input[0])
         free(self->rs_input[0]);
     if (self->rs_input[1])
