@@ -110,6 +110,9 @@ static struct peakfilter *str_pf_l, *str_pf_r;
 static unsigned int port_connection_count;
 /* counts the number of times port connection counts have been reported */
 static unsigned int port_reports;
+/* voip pan/downmix related */
+static int voip_pan_f;
+static float voip_pan_l, voip_pan_r;
 
 static jack_nframes_t alarm_size;
 
@@ -170,7 +173,7 @@ static char *flag;
 static char *channel_mode_string;
 static char *use_jingles_vol_2;
 static char *jackport, *jackport2, *jackfilter;
-static char *effect_ix;
+static char *effect_ix, *voip_pan;
 static char *session_event_string, *session_commandline;
 
 static struct smoothing_volume jingles_headroom_smoothing;
@@ -217,6 +220,7 @@ static struct kvpdict kvpdict[] = {
             { "JPRT", &jackport, NULL },
             { "JPT2", &jackport2, NULL },
             { "EFCT", &effect_ix, NULL },
+            { "VPAN", &voip_pan, NULL },
             { "ACTN", &action, NULL },                   /* Action to take */
             { "session_event", &session_event_string, NULL },
             { "session_command", &session_commandline, NULL },
@@ -433,8 +437,8 @@ int mixer_process_audio(jack_nframes_t nframes, void *arg)
     float df;           /* main player ducking factor */
     float idf;          /* interlude player ducking factor */
     /* the following are used to calculate the microphone mix */
-    sample_t lc_s_micmix = 0.0f, rc_s_micmix = 0.0f, d_micmix = 0.0f;
-    sample_t lc_s_auxmix = 0.0f, rc_s_auxmix = 0.0f;
+    sample_t lc_s_micmix, rc_s_micmix, dl_micmix, dr_micmix;
+    sample_t lc_s_auxmix, rc_s_auxmix;
     /* the following are used to apply the output of the compressor code to the audio levels */
     sample_t compressor_gain = 1.0;
     /* a counter variable used to trigger the volume smoothing on a regular basis */
@@ -512,7 +516,7 @@ int mixer_process_audio(jack_nframes_t nframes, void *arg)
         pthread_mutex_unlock(&midi_mutex);
         }
 
-    /* get the data for the jack ports */
+    /* get the data pointers for the jack ports */
     {
         struct jack_ports *p = &g.port;
         
@@ -561,8 +565,8 @@ int mixer_process_audio(jack_nframes_t nframes, void *arg)
     xlplayer_read_start_all(players, nframes, players_roster);
     xlplayer_read_start_all(plr_j, nframes, plr_j_roster);
     
-    /* there are four mixer modes and the only seemingly efficient way to do them is */
-    /* to basically copy a lot of code four times over hence the huge size */
+    /* there are four mixer modes with a lot of shared code */
+    /* to keep things smaller and more maintainable macros have been used */
     if (simple_mixer == FALSE && mixermode == NO_PHONE)  /* Fully featured mixer code */
         {
         memset(lps_buffer, 0, nframes * sizeof (sample_t)); /* send silence to VOIP */
@@ -576,13 +580,14 @@ int mixer_process_audio(jack_nframes_t nframes, void *arg)
                 update_smoothed_volumes();
                 
             df = powf(mic_process_all(mics), dfmod);
-            for (micp = mics, lc_s_micmix = rc_s_micmix = lc_s_auxmix = rc_s_auxmix = d_micmix = 0.0f; *micp; micp++)
+            for (micp = mics, lc_s_micmix = rc_s_micmix = lc_s_auxmix = rc_s_auxmix = dl_micmix = dr_micmix = 0.0f; *micp; micp++)
                 {
                 lc_s_micmix += (*micp)->mlcm;
                 rc_s_micmix += (*micp)->mrcm;
                 lc_s_auxmix += (*micp)->alcm;
                 rc_s_auxmix += (*micp)->arcm;
-                d_micmix += (*micp)->munpmdj;
+                dl_micmix += (*micp)->lmunpmdj;
+                dr_micmix += (*micp)->rmunpmdj;
                 }
          
             /* ducking calculation */
@@ -665,8 +670,8 @@ int mixer_process_audio(jack_nframes_t nframes, void *arg)
 
             if (stream_monitor == FALSE)
                 {
-                *lap = ((plr_l->ls_aud + plr_r->ls_aud) * *jh + e_ls) * df + d_micmix + lc_s_auxmix + plr_i->ls_aud * idf * *jhi;
-                *rap = ((plr_l->rs_aud + plr_r->rs_aud) * *jh + e_rs) * df + d_micmix + rc_s_auxmix + plr_i->rs_aud * idf * *jhi;
+                *lap = ((plr_l->ls_aud + plr_r->ls_aud) * *jh + e_ls) * df + dl_micmix + lc_s_auxmix + plr_i->ls_aud * idf * *jhi;
+                *rap = ((plr_l->rs_aud + plr_r->rs_aud) * *jh + e_rs) * df + dr_micmix + rc_s_auxmix + plr_i->rs_aud * idf * *jhi;
                 compressor_gain = db2level(limiter(&audio_limiter, *lap, *rap));
                 *lap *= compressor_gain;
                 *rap *= compressor_gain;
@@ -729,13 +734,14 @@ int mixer_process_audio(jack_nframes_t nframes, void *arg)
                     update_smoothed_volumes();
             
                 mic_process_all(mics);
-                for (micp = mics, lc_s_micmix = rc_s_micmix = lc_s_auxmix = rc_s_auxmix = d_micmix = 0.0f; *micp; micp++)
+                for (micp = mics, lc_s_micmix = rc_s_micmix = lc_s_auxmix = rc_s_auxmix = dl_micmix = dr_micmix = 0.0f; *micp; micp++)
                     {
                     lc_s_micmix += (*micp)->mlcm;
                     rc_s_micmix += (*micp)->mrcm;
                     lc_s_auxmix += (*micp)->alcm;
                     rc_s_auxmix += (*micp)->arcm;
-                    d_micmix += (*micp)->munpmdj;
+                    dl_micmix += (*micp)->lmunpmdj;
+                    dr_micmix += (*micp)->rmunpmdj;
                     }
 
                 /* No ducking but headroom still must apply */
@@ -753,6 +759,13 @@ int mixer_process_audio(jack_nframes_t nframes, void *arg)
                 compressor_gain = db2level(limiter(&incoming_phone_limiter, *lprp *= voip_lc_aud, *rprp *= voip_rc_aud));
                 *lprp *= compressor_gain;
                 *rprp *= compressor_gain;
+                if (voip_pan_f)
+                    {
+                    float dnmix = (*lprp + *rprp) / 2.0f;
+                    
+                    *lprp = dnmix * voip_pan_l;
+                    *rprp = dnmix * voip_pan_r;
+                    }
 
                 /* The main mix */
                 *dolp = (plr_l->ls_str + plr_r->ls_str) * *jh * df + *lprp + *lpsp + lc_s_auxmix + plr_i->ls_str * idf * *jhi;
@@ -768,8 +781,8 @@ int mixer_process_audio(jack_nframes_t nframes, void *arg)
 
                 if (stream_monitor == FALSE)
                     {
-                    *lap = (plr_l->ls_aud + plr_r->ls_aud) * *jh * df + *lprp + lc_s_auxmix + plr_i->ls_aud * idf * *jhi + d_micmix + e_ls;
-                    *rap = (plr_l->rs_aud + plr_r->rs_aud) * *jh * df + *rprp + rc_s_auxmix + plr_i->rs_aud * idf * *jhi + d_micmix + e_rs;
+                    *lap = (plr_l->ls_aud + plr_r->ls_aud) * *jh * df + *lprp + lc_s_auxmix + plr_i->ls_aud * idf * *jhi + dl_micmix + e_ls;
+                    *rap = (plr_l->rs_aud + plr_r->rs_aud) * *jh * df + *rprp + rc_s_auxmix + plr_i->rs_aud * idf * *jhi + dr_micmix + e_rs;
                     compressor_gain = db2level(limiter(&audio_limiter, *lap, *rap));
                     *lap *= compressor_gain;
                     *rap *= compressor_gain;
@@ -797,13 +810,14 @@ int mixer_process_audio(jack_nframes_t nframes, void *arg)
                         update_smoothed_volumes();
 
                     mic_process_all(mics);
-                    for (micp = mics, lc_s_micmix = rc_s_micmix = lc_s_auxmix = rc_s_auxmix = d_micmix = 0.0f; *micp; micp++)
+                    for (micp = mics, lc_s_micmix = rc_s_micmix = lc_s_auxmix = rc_s_auxmix = dl_micmix = dr_micmix = 0.0f; *micp; micp++)
                         {
                         lc_s_micmix += (*micp)->mlc;
                         rc_s_micmix += (*micp)->mrc;
                         lc_s_auxmix += (*micp)->alcm;
                         rc_s_auxmix += (*micp)->arcm;
-                        d_micmix += (*micp)->munpm;
+                        dl_micmix += (*micp)->lmunpm;
+                        dr_micmix += (*micp)->rmunpm;
                         }
                     
                     /* No ducking */
@@ -829,13 +843,20 @@ int mixer_process_audio(jack_nframes_t nframes, void *arg)
                     compressor_gain = db2level(limiter(&incoming_phone_limiter, *lprp *= voip_lc_aud, *rprp *= voip_rc_aud));
                     *lprp *= compressor_gain;
                     *rprp *= compressor_gain;
+                    if (voip_pan_f)
+                        {
+                        float dnmix = (*lprp + *rprp) / 2.0f;
+                        
+                        *lprp = dnmix * voip_pan_l;
+                        *rprp = dnmix * voip_pan_r;
+                        }
                     
                     COMMON_MIX2();
 
                     if (stream_monitor == FALSE) /* the DJ can hear the VOIP phone call */
                         {
-                        *lap = (*lsp * mb_lc_aud) + e_ls + d_micmix + (lc_s_auxmix *mb_lc_aud) + *lprp;
-                        *rap = (*rsp * mb_lc_aud) + e_rs + d_micmix + (rc_s_auxmix *mb_rc_aud) + *rprp;
+                        *lap = (*lsp * mb_lc_aud) + e_ls + dl_micmix + (lc_s_auxmix *mb_lc_aud) + *lprp;
+                        *rap = (*rsp * mb_lc_aud) + e_rs + dr_micmix + (rc_s_auxmix *mb_rc_aud) + *rprp;
                         compressor_gain = db2level(limiter(&audio_limiter, *lap, *rap));
                         *lap *= compressor_gain;
                         *rap *= compressor_gain;
@@ -863,13 +884,14 @@ int mixer_process_audio(jack_nframes_t nframes, void *arg)
                             update_smoothed_volumes();
 
                         df = powf(mic_process_all(mics), dfmod);
-                        for (micp = mics, lc_s_micmix = rc_s_micmix = lc_s_auxmix = rc_s_auxmix = d_micmix = 0.0f; *micp; micp++)
+                        for (micp = mics, lc_s_micmix = rc_s_micmix = lc_s_auxmix = rc_s_auxmix = dl_micmix = dr_micmix = 0.0f; *micp; micp++)
                             {
                             lc_s_micmix += (*micp)->mlcm;
                             rc_s_micmix += (*micp)->mrcm;
                             lc_s_auxmix += (*micp)->alcm;
                             rc_s_auxmix += (*micp)->arcm;
-                            d_micmix += (*micp)->munpmdj;
+                            dl_micmix += (*micp)->lmunpmdj;
+                            dr_micmix += (*micp)->rmunpmdj;
                             }
 
                         /* ducking calculation */
@@ -898,8 +920,8 @@ int mixer_process_audio(jack_nframes_t nframes, void *arg)
 
                         if (stream_monitor == FALSE)
                             {
-                            *lap = ((plr_l->ls_aud + plr_r->ls_aud) * *jh + e_ls) * df + d_micmix + lc_s_auxmix + plr_i->ls_aud * idf * *jhi;
-                            *rap = ((plr_l->rs_aud + plr_r->rs_aud) * *jh + e_ls) * df + d_micmix + rc_s_auxmix + plr_i->rs_aud * idf * *jhi;
+                            *lap = ((plr_l->ls_aud + plr_r->ls_aud) * *jh + e_ls) * df + dl_micmix + lc_s_auxmix + plr_i->ls_aud * idf * *jhi;
+                            *rap = ((plr_l->rs_aud + plr_r->rs_aud) * *jh + e_ls) * df + dr_micmix + rc_s_auxmix + plr_i->rs_aud * idf * *jhi;
                             compressor_gain = db2level(limiter(&audio_limiter, *lap, *rap));
                             *lap *= compressor_gain;
                             *rap *= compressor_gain;
@@ -1102,8 +1124,8 @@ void mixer_init(void)
     int n = 0;
     int ne = atoi(getenv("num_effects"));
 
-    if(! ((players[n++] = plr_l = xlplayer_create(sr, MAIN_RB_SIZE, "left", &g.app_shutdown, &volume, 0, &left_stream, &left_audio, 0.1f)) &&
-            (players[n++] = plr_r = xlplayer_create(sr, MAIN_RB_SIZE, "right", &g.app_shutdown, &volume2, 0, &right_stream, &right_audio, 0.1f))))
+    if(! ((players[n++] = plr_l = xlplayer_create(sr, MAIN_RB_SIZE, "left", &g.app_shutdown, &volume, 0, &left_stream, &left_audio, 0.3f)) &&
+            (players[n++] = plr_r = xlplayer_create(sr, MAIN_RB_SIZE, "right", &g.app_shutdown, &volume2, 0, &right_stream, &right_audio, 0.3f))))
         {
         fprintf(stderr, "failed to create main player modules\n");
         exit(5);
@@ -1133,7 +1155,7 @@ void mixer_init(void)
         plr_j[i]->fade_mode = 3;
         }
     
-    if (!(players[n++] = plr_i = xlplayer_create(sr, MAIN_RB_SIZE, "interlude", &g.app_shutdown, &interludevol, 0, &inter_stream, &inter_audio, 0.25f)))
+    if (!(players[n++] = plr_i = xlplayer_create(sr, MAIN_RB_SIZE, "interlude", &g.app_shutdown, &interludevol, 0, &inter_stream, &inter_audio, 0.3f)))
         {
         fprintf(stderr, "failed to create interlude player module\n");
         exit(5);
@@ -1420,6 +1442,23 @@ int mixer_main()
         speex_tag_write(speexpathname, speexcreatedby, speextaglist);
 #endif
 
+    if (!strcmp(action, "voippan"))
+        {
+        int voippanval = atoi(voip_pan);
+        
+        if (voippanval == -1)
+            voip_pan_f = 0;
+        else
+            {
+            double x = voippanval * M_PI_2 / 100.0;
+            
+            voip_pan_l = (float)cos(x);
+            voip_pan_r = (float)sin(x);
+            
+            voip_pan_f = 1;
+            }
+        }
+
     if (!strcmp(action, "mixstats"))
         {
         if(sscanf(mixer_string,
