@@ -17,12 +17,15 @@
 #   along with this program in the file entitled COPYING.
 #   If not, see <http://www.gnu.org/licenses/>.
 
+from __future__ import absolute_import
+
 import re
 import json
 import time
 import sys
 import threading
 import gettext
+from inspect import getargspec
 from functools import wraps, partial
 
 import gobject
@@ -30,18 +33,19 @@ import gtk
 import pango
 
 try:
-    import irclib
+    from irc import client
+    from irc import events
 except ImportError:
-    HAVE_IRCLIB = False
+    HAVE_IRC = False
 else:
-    HAVE_IRCLIB = True
+    HAVE_IRC = True
 
 from idjc import FGlobs
 from idjc.prelims import ProfileManager
 from .gtkstuff import DefaultEntry
 from .gtkstuff import NamedTreeRowReference
 from .gtkstuff import ConfirmationDialog
-from .gtkstuff import threadslock
+from .gtkstuff import threadslock, gdklock
 from .utils import string_multireplace
 from .tooltips import set_tip
 
@@ -402,13 +406,6 @@ class ServerDialog(gtk.Dialog):
         self.network.set_width_chars(25)
         self.hostname = gtk.Entry()
         self.port = gtk.SpinButton(server_port_adj)
-        # TC: Checkbutton label text regarding SSL security protocol.
-        self.ssl = gtk.CheckButton(_("SSL"))
-        set_tip(self.ssl, 
-            _("Connect to the server using the SSL security protocol.\n\n"
-            "This feature is typically offered on different ports than the "
-            "standard connection method and is widespread but not universally "
-            "available."))
         self.username = gtk.Entry()
         self.password = gtk.Entry()
         self.password.set_visibility(False)
@@ -442,7 +439,7 @@ class ServerDialog(gtk.Dialog):
                             # TC: label for hostname entry.
                             _("Hostname"),
                             # TC: TCP/IP port number label.
-                            _("Port"), "",
+                            _("Port"),
                             _("User name"),
                             _("Password"), "",
                             # TC: IRC nickname data entry label.
@@ -455,7 +452,7 @@ class ServerDialog(gtk.Dialog):
                             _("Real name"),
                             # TC: The NickServ password.
                             _("NickServ p/w")),
-                (self.network, self.hostname, self.port, self.ssl,
+                (self.network, self.hostname, self.port,
                  self.username, self.password, self.manual_start, self.nick1,
                  self.nick2, self.nick3, self.realname, self.nickserv))):
             # TC: Tooltip to IRC 'User name' field.
@@ -488,7 +485,7 @@ class ServerDialog(gtk.Dialog):
         """Data extraction method."""
 
         return (self.manual_start.get_active(),
-            self.port.get_value(), self.ssl.get_active(),
+            self.port.get_value(), False,
             self.network.get_text().strip(), self.hostname.get_text().strip(),
             self.username.get_text().strip(), self.password.get_text().strip(),
             self.nick1.get_text().strip(), self.nick2.get_text().strip(),
@@ -520,7 +517,7 @@ class EditServerDialog(ServerDialog, EditDialogMixin):
         n = iter(orig_data).next
         self.manual_start.set_active(n())
         self.port.set_value(n())
-        self.ssl.set_active(n())
+        n()
         self.network.set_text(n())
         self.hostname.set_text(n())
         self.username.set_text(n())
@@ -811,7 +808,7 @@ class IRCRowReference(NamedTreeRowReference):
     """
     
     _lookup = {
-        1: {"manual":2, "port":3, "ssl":4, "network":5, "hostname":6, "username":7,
+        1: {"manual":2, "port":3, "unused":4, "network":5, "hostname":6, "username":7,
              "password":8, "nick1":9, "nick2":10, "nick3":11, "realname":12,
              "nickserv":13, "nick":14},
 
@@ -923,14 +920,14 @@ class IRCPane(gtk.VBox):
         selection.connect("changed", self._on_selection_changed, edit, new)
         selection.select_path(0)
 
-        if HAVE_IRCLIB:
+        if HAVE_IRC:
             self.pack_start(sw)
             self.pack_start(bb, False)
             self.connections_controller = ConnectionsController(self._treestore)
         else:
             self.set_sensitive(False)
             label = gtk.Label(
-                _("This feature requires the installation of python-irclib."))
+                _("This feature requires the installation of python-irc."))
             self.add(label)
             self.connections_controller = ConnectionsController(None)
 
@@ -947,7 +944,7 @@ class IRCPane(gtk.VBox):
     def marshall(self):
         """Convert all our data into a string."""
 
-        if HAVE_IRCLIB:
+        if HAVE_IRC:
             store = [self._m_signature()]
             self._treestore.foreach(self._m_read, store)
             return json.dumps(store)
@@ -955,7 +952,7 @@ class IRCPane(gtk.VBox):
             return ""
 
     def _m_read(self, model, path, iter, store):
-        row = IRCRowReference(model[path])
+        row = IRCRowReference(list(model[path]))
         if row.type == 1 and row.active and row.manual:
             row.active = 0
 
@@ -964,7 +961,7 @@ class IRCPane(gtk.VBox):
     def unmarshall(self, data):
         """Set the TreeStore with data from a string."""
         
-        if HAVE_IRCLIB:
+        if HAVE_IRC:
             try:
                 store = json.loads(data)
             except ValueError:
@@ -1019,9 +1016,6 @@ class IRCPane(gtk.VBox):
                     text += "(%s)" % row.network
 
                 opt = []
-                if row.ssl:
-                    # TC: Indicator text: We are using SSL protocol.
-                    opt.append(_("SSL"))
                 if row.password:
                     # TC: Indicator text: We used a password.
                     opt.append(_("PASSWORD"))
@@ -1190,7 +1184,7 @@ class IRCConnection(gtk.TreeRowReference, threading.Thread):
         self._keepalive = True
         self._have_welcome = False
         self._stream_active = stream_active
-        self.irc = irclib.IRC()
+        self.irc = client.IRC()
         self.server = self.irc.server()
         self.start()
         self._hooks.append((model, model.connect("row-inserted",
@@ -1237,7 +1231,7 @@ class IRCConnection(gtk.TreeRowReference, threading.Thread):
                         each = each.split(":")
                         try:
                             channel, key = each
-                        except:
+                        except ValueError:
                             channel = each[0]
                             key = ""
                         self.server.join(channel, key)
@@ -1254,8 +1248,8 @@ class IRCConnection(gtk.TreeRowReference, threading.Thread):
 
     def _on_privmsg_ready(self, handler, targets, message, delay):
         if self._have_welcome:
-            chan_targets = filter(lambda x: x[0] in "#&", targets)
-            user_targets = filter(lambda x: x[0] not in "#&", targets)
+            chan_targets = [x.split(":")[0] for x in targets if x[0] in "#&"]
+            user_targets = [x for x in targets if x[0] not in "#&"]
 
             def deferred():
                 self.server.privmsg_many(chan_targets, message)
@@ -1272,42 +1266,63 @@ class IRCConnection(gtk.TreeRowReference, threading.Thread):
         if path == self.get_path():
             row = self.get_model()[self.get_path()]
             if model.path_is_active(path):
+                ref = gtk.TreeRowReference(model, path)
                 hostname = row.hostname
                 port = row.port
                 nickname = row.nick1 or "eyedeejaycee"
                 password = row.password or None
                 username = row.username or None
                 ircname = row.realname or None
-                ssl = bool(row.ssl)
+                opts = {}
                 def deferred():
-                    try:
-                        self._alternates = [
+                    self._alternates = [
                             row.nick2, row.nick3, nickname + "_",
                             row.nick2 + "_", row.nick3 + "_", nickname + "__",
                             row.nick2 + "__", row.nick3 + "__"]
+                            
+                    connect = partial(self.server.connect, hostname, port,
+                                        nickname, password, username, ircname)
+                                        
+                    def try_connect(*delays):
+                        model = ref.get_model()
+                        path = ref.get_path()
+                        if not ref.valid() or not model.path_is_active(path):
+                            print "IRC connection attempt cancelled"
+                            return
 
-                        self.server.connect(hostname, port, nickname, password,
-                                                    username, ircname, ssl=ssl)
-                    except irclib.ServerConnectionError, e:
-                        self._ui_set_nick("")
-                        print >>sys.stderr, str(e) + " %s@%s:%d" % (
-                                                    nickname, hostname, port) 
-                    else:
-                        self._ui_set_nick(nickname)
-                        print "New IRC connection: %s@%s:%d" % (
+                        print "Attempting to connect IRC %s:%d" % (
+                                                            hostname, port)
+                        try:
+                            connect()
+                        except client.ServerConnectionError as e:
+                            print e
+                            try:
+                                delay = delays[0]
+                            except IndexError:
+                                print "No more connection attempts"
+                                self._ui_set_nick("")
+                            else:
+                                print "%d more tries" % len(delays)
+                                self.server.execute_delayed(delay, try_connect,
+                                                                    delays[1:])
+                        else:
+                            self._ui_set_nick(nickname)
+                            print "New IRC connection: %s@%s:%d" % (
                                                     nickname, hostname, port)
+                                                    
+                    try_connect(1, 2, 3)
             else:
                 def deferred():
                     try:
                         self.server.disconnect()
-                    except irclib.ServerConnectionError, e:
+                    except client.ServerConnectionError as e:
                         print >>sys.stderr, str(e)
                     self._ui_set_nick("")
 
             self._queue.append(deferred)
 
     def run(self):
-        for event in irclib.all_events:
+        for event in events.all:
             try:
                 target = getattr(self, "_on_" + event)
             except AttributeError:
@@ -1319,29 +1334,31 @@ class IRCConnection(gtk.TreeRowReference, threading.Thread):
                 self._queue.pop(0)()
             
             self.irc.process_once(0.2)
+        
+        self.irc.process_once()
 
     def cleanup(self):
-        def deferred():
-            try:
-                self.server.disconnect()
-            except irclib.ServerConnectionError, e:
-                print >>sys.stderr, str(e)
-        self._queue.append(deferred)
-        
         for each in self._message_handlers:
             each.cleanup()
         for obj, handler_id in self._hooks:
             obj.disconnect(handler_id)
 
-        self.server.add_global_handler("disconnect",
-                                                self.thread_purge_on_disconnect)
-        if not self.server.is_connected():
-            self._keepalive = False
-        self.join(1.0)
-        if self.is_alive():
-            self.server.close()
+        if self.server.is_connected():
+            def deferred():
+                self.server.add_global_handler("disconnect", self.end_thread)
+                try:
+                    self.server.disconnect()
+                except client.ServerConnectionError as e:
+                    print >>sys.stderr, str(e)
+                self._ui_set_nick("")
 
-    def thread_purge_on_disconnect(self, server, event):
+            self._queue.append(deferred)
+        else:
+            self._keepalive = False
+        
+        self.join(1.0)
+
+    def end_thread(self, server, event):
         self._keepalive = False
 
     @threadslock
@@ -1364,7 +1381,7 @@ class IRCConnection(gtk.TreeRowReference, threading.Thread):
 
     @threadslock
     def _on_welcome(self, server, event):
-        print "Got IRC welcome", event.source()
+        print "Got IRC welcome", event.source
         self._have_welcome = True
         self._channels_invalidate()
         model = self.get_model()
@@ -1375,12 +1392,12 @@ class IRCConnection(gtk.TreeRowReference, threading.Thread):
             iter = model.iter_next(iter)
         row = model[path]
         model.row_changed_block()
-        row.nick = event.target()
+        row.nick = event.target
         model.row_changed_unblock()
         
         target = row.nick1
         nspw = row.nickserv
-        if event.target() != target and nspw:
+        if event.target != target and nspw:
             self._nick_recover(server, target, nspw)
 
     def _nick_recover(self, server, target, nspw):
@@ -1395,34 +1412,35 @@ class IRCConnection(gtk.TreeRowReference, threading.Thread):
             server.execute_delayed(i, func, args)
 
     def _on_privnotice(self, server, event):
-        source = event.source()
+        source = event.source
         if source is not None:
-            source = source.split("!")[0]
+            source = source.split("@")[0]
             
-            print "-%s- %s" % (source, event.arguments()[0])
+            if source != "Global!services":
+                print "-%s- %s" % (source, event.arguments[0])
+                
             if source == "NickServ!services":
-                gtk.gdk.threads_enter()
-                nspw = self.get_model()[self.get_path()].nickserv
-                gtk.gdk.threads_leave()
+                with gdklock():
+                    nspw = self.get_model()[self.get_path()].nickserv
 
-                if "NickServ IDENTIFY" in event.arguments()[0] and nspw:
+                if "NickServ IDENTIFY" in event.arguments[0] and nspw:
                     server.privmsg("NickServ", "IDENTIFY %s" % nspw)
                     print "Issued IDENTIFY command to NickServ"
-                    self._ui_set_nick(event.target())
-                elif "Guest" in event.arguments()[0]:
-                    newnick = event.arguments()[0].split()[-1].strip(ASCII_C0)
+                    self._ui_set_nick(event.target)
+                elif "Guest" in event.arguments[0]:
+                    newnick = event.arguments[0].split()[-1].strip(ASCII_C0)
                     self._ui_set_nick(newnick)
                     if nspw:
-                        self._nick_recover(server, event.target(), nspw)
+                        self._nick_recover(server, event.target, nspw)
                 else:
-                    self._ui_set_nick(event.target())
+                    self._ui_set_nick(event.target)
         else:
             self._generic_handler(server, event)
 
     def _on_disconnect(self, server, event):
         self._have_welcome = False
         self._ui_set_nick("")
-        print event.source(), "disconnected"
+        print event.source, "disconnected"
 
     def _on_nicknameinuse(self, server, event):
         self._try_alternate_nick()
@@ -1437,11 +1455,11 @@ class IRCConnection(gtk.TreeRowReference, threading.Thread):
         self._try_alternate_nick()
 
     def _on_join(self, server, event):
-        print "Channel joined", event.target()
+        print "Channel joined", event.target
 
     def _on_ctcp(self, server, event):
-        source = event.source().split("!")[0]
-        args = event.arguments()
+        source = event.source.split("!")[0]
+        args = event.arguments
         reply = partial(server.ctcp_reply, source)
         
         if args == ["CLIENTINFO"]:
@@ -1449,7 +1467,7 @@ class IRCConnection(gtk.TreeRowReference, threading.Thread):
                                             "PLAYED STREAMSTATUS KILLSTREAM")
         
         elif args == ["VERSION"]:
-            reply("VERSION %s %s (python-irclib)" % (
+            reply("VERSION %s %s (python-irc)" % (
                                 FGlobs.package_name, FGlobs.package_version))
         elif args == ["TIME"]:
             reply("TIME " + time.ctime())
@@ -1462,9 +1480,8 @@ class IRCConnection(gtk.TreeRowReference, threading.Thread):
             
         elif args == ["PLAYED"]:
             t = time.time()
-            gtk.gdk.threads_enter()
-            show = [x for x in self._played if t - x[1] < 5400.0]
-            gtk.gdk.threads_leave()
+            with gdklock():
+                show = [x for x in self._played if t - x[1] < 5400.0]
 
             for i, each in enumerate(show, start=1):
                 age = int((t - each[1]) // 60)
@@ -1650,7 +1667,7 @@ class MessageHandlerForType_5(MessageHandler):
     @threadslock
     def _timeout(self):
         self.issue_messages(partial(self._delay_calc,
-                                                    the_time=int(time.time())))
+                                                the_time=int(time.time())))
         return True
         
     def _delay_calc(self, row, the_time):
