@@ -25,6 +25,7 @@ import uuid
 import gtk
 import gobject
 import itertools
+import urllib
 
 from idjc import *
 from .playergui import *
@@ -53,7 +54,14 @@ class Effect(gtk.HBox):
     L.E.D., stop, and config button.
     """
 
-    dndtargets = [("IDJC_EFFECT_BUTTON", gtk.TARGET_SAME_APP, 6)]
+    dndsources = (("IDJC_EFFECT_BUTTON", gtk.TARGET_SAME_APP, 6),)
+
+    dndtargets = (  # Drag and drop source target specs.
+        ('text/plain', 0, 1),
+        ('TEXT', 0, 2),
+        ('STRING', 0, 3),
+        ('text/uri-list', 0, 4),
+        ("IDJC_EFFECT_BUTTON", gtk.TARGET_SAME_APP, 6))
 
     def __init__(self, num, others, parent):
         self.num = num
@@ -100,7 +108,7 @@ class Effect(gtk.HBox):
         self.pack_start(pvbox, False)
         self.trigger.connect("clicked", self._on_trigger)
         self.trigger.drag_dest_set(gtk.DEST_DEFAULT_ALL,
-            self.dndtargets, gtk.gdk.ACTION_DEFAULT | gtk.gdk.ACTION_MOVE)
+            self.dndtargets, gtk.gdk.ACTION_DEFAULT | gtk.gdk.ACTION_COPY)
         self.trigger.connect("drag-data-received", self._drag_data_received)
         set_tip(self.trigger, _('Play'))
         
@@ -120,8 +128,10 @@ class Effect(gtk.HBox):
         self.pack_start(self.config, False)
         self.config.connect("clicked", self._on_config)
         self.config.drag_source_set(gtk.gdk.BUTTON1_MASK,
-            self.dndtargets, gtk.gdk.ACTION_DEFAULT | gtk.gdk.ACTION_MOVE)
+            self.dndsources, gtk.gdk.ACTION_DEFAULT | gtk.gdk.ACTION_COPY)
+        self.config.connect("drag-begin", self._drag_begin)
         self.config.connect("drag-data-get", self._drag_get_data)
+        self.config.connect("drag-end", self._drag_end)
         set_tip(self.config, _('Configure'))
 
         self.dialog = EffectConfigDialog(self, parent.window)
@@ -149,21 +159,35 @@ class Effect(gtk.HBox):
         vb.pack_start(self.tabeffectprog)
         self.tabwidget.show_all()
 
+    def _drag_begin(self, widget, context):
+        widget.drag_highlight()
+        context.set_icon_stock(gtk.STOCK_PROPERTIES, -5, -5)
+        
+    def _drag_end(self, widget, context):
+        widget.drag_unhighlight()
+
     def _drag_get_data(self, widget, context, selection, target_id, etime):
         selection.set(selection.target, 8, str(self.num))
         return True
 
     def _drag_data_received(self, widget, context, x, y, dragged, info, etime):
-        other = self.others[int(dragged.data)]
-        if context.action == gtk.gdk.ACTION_MOVE:
-            if other == self:
-                context.finish(False, False, etime)
-            else:
+        if context.targets == ["IDJC_EFFECT_BUTTON"]:
+            other = self.others[int(dragged.data)]
+            if other != self:
                 self.stop.clicked()
                 other.stop.clicked()
-                context.finish(True, False, etime)
                 self._swap(other)
-        return True
+                return True
+        else:
+            data = dragged.data.splitlines()
+            if len(data) == 1 and data[0].startswith("file://"):
+                pathname = urllib.unquote(data[0][7:])
+                title = self.interlude.get_media_metadata(pathname).title
+                if title:
+                    self.stop.clicked()
+                    self._set(pathname, title, 0.0)
+                    return True
+        return False
         
     def _swap(self, other):
         new_pathname = other.pathname
@@ -204,6 +228,8 @@ class Effect(gtk.HBox):
                 self.tabeffecttime.set_text('0.0')
                 self.tabeffectprog.set_fraction(0.0)
                 self.approot.jingles.nb_effects_box.pack_start(self.tabwidget)
+                self.approot.effect_started(self.trigger_label.get_text(),
+                                                    self.pathname, self.num)
             else: # Restarted the effect
                 self.effect_start = time.time()
             self.approot.mixer_write(
@@ -236,6 +262,7 @@ class Effect(gtk.HBox):
             self.timeout_source_id = None
             self.progress.set_fraction(0.0)
             self.approot.jingles.nb_effects_box.remove(self.tabwidget)
+            self.approot.effect_stopped(self.num)
 
     def _on_dialog_response(self, dialog, response_id, pathname=None):
         if response_id in (gtk.RESPONSE_ACCEPT, gtk.RESPONSE_NO):
@@ -335,10 +362,30 @@ class EffectConfigDialog(gtk.FileChooserDialog):
         vbox.pack_start(hbox, False)
         
         ca.show_all()
+        self.connect("notify::visible", self._cb_notify_visible)
         self.connect("delete-event", lambda w, e: w.hide() or True)
         self.connect("response", self._cb_response)
         self.add_filter(self.file_filter)
 
+    def set_filename(self, filename):
+        self._stored_filename = filename
+        gtk.FileChooserDialog.set_filename(self, filename)
+
+    def _cb_notify_visible(self, *args):
+        # Make sure filename is shown in the location box.
+
+        if self.get_visible():
+            filename = self.get_filename()
+            if filename is None:
+                try:
+                    if self._stored_filename is not None:
+                        self.set_filename(self._stored_filename)
+                except AttributeError:
+                    pass
+        else:
+            self._stored_filename = self.get_filename()
+                    
+
     def _cb_response(self, dialog, response_id):
         dialog.hide()
         if response_id == gtk.RESPONSE_NO:
