@@ -15,8 +15,6 @@
 #   along with this program in the file entitled COPYING.
 #   If not, see <http://www.gnu.org/licenses/>.
 
-__all__ = ['mixprefs']
-
 
 import os
 import shutil
@@ -24,16 +22,19 @@ import gettext
 import itertools
 
 import gtk
+import glib
 
 from idjc import FGlobs, PGlobs
 from . import licence_window
 from . import songdb
 from . import midicontrols
-from .gtkstuff import WindowSizeTracker, DefaultEntry
+from .gtkstuff import WindowSizeTracker, DefaultEntry, threadslock
 from .prelims import ProfileManager
 from .utils import PathStr
 from .tooltips import set_tip, MAIN_TIPS
 
+__all__ = ['mixprefs', 'PanPresetChooser']
+
 
 _ = gettext.translation(FGlobs.package_name, FGlobs.localedir,
                                                         fallback=True).gettext
@@ -58,7 +59,6 @@ class CSLEntry(gtk.Entry):
         self.connect("key-press-event", self.cb_keypress)
 
 
-
 class InitialPlayerConfig(gtk.Frame):
     def __init__(self, title, player, prefix):
         self.player = player
@@ -104,7 +104,6 @@ class InitialPlayerConfig(gtk.Frame):
             prefix + "todj": self.to_dj
         }
 
-
     def apply(self):
         p = self.player
         
@@ -116,6 +115,157 @@ class InitialPlayerConfig(gtk.Frame):
             p.digiprogress_click()
 
 
+class PanWidget(gtk.Frame):
+    _instances = []
+    
+    def __init__(self, title, commandname):
+        self._instances.append(self)
+        
+        gtk.Frame.__init__(self)
+        self.modes = (1, 2, 3)
+        set_tip(self, _('Stereo panning is the selection of where an audio '
+        'source sits from left to right within the stereo mix.\n\nThis control '
+        'maintains constant audio power throughout its range of travel, giving '
+        '-3dB attenuation in both audio channels at the half way point.\n\n'
+        'If you require 0dB straight down the middle or require a stereo '
+        'source remain as stereo then this feature should be turned off.\n\n'
+        'Paired channels should be set to 100% left/right unless narrowing of '
+        'the stereo field is the intention.'))
+        self.valuesdict = {}
+        self.activedict = {}
+        self._source_id = None
+        
+        hbox = gtk.HBox()
+        self.pan_active = gtk.CheckButton(title)
+        self.activedict[commandname + "_pan_active"] = self.pan_active
+        hbox.pack_start(self.pan_active, False, False, 0)
+        self.pan_active.show()
+        self.set_label_widget(hbox)
+        hbox.show()
+        
+        panvbox = gtk.VBox()
+        panvbox.set_border_width(1)
+        self.add(panvbox)
+
+        panadj = gtk.Adjustment(50.0, 0.0, 100.0, 1, 1, 0)
+        self.pan = gtk.HScale(panadj)
+        self.pan.set_digits(0)
+        self.pan.connect("format-value", self._cb_format_value)
+        self.valuesdict[commandname + "_pan"] = self.pan
+        panvbox.pack_start(self.pan, False, padding=2)
+        
+        label = gtk.Label(_('Presets'))
+        label.set_alignment(0.0, 0.5)
+        label.set_padding(3, 3)
+        panvbox.pack_start(label, False)
+        
+        self._presets = []
+        for i in range(PGlobs.num_panpresets):
+            preadj = gtk.Adjustment(50.0, 0.0, 100.0, 1, 1, 0)
+            preset = gtk.HScale(preadj)
+            preset.set_digits(0)
+            preset.connect("format-value", self._cb_format_value)
+            self.valuesdict[commandname + "_panpreset" + str(i)] = preset
+            self._presets.append(preset)
+            panvbox.pack_start(preset, False, padding=2)
+            
+    def load_preset(self, index):
+        try:
+            target = int(self._presets[index].get_value() + 0.5)
+        except IndexError:
+            print "Attempt made to load a non existent pan preset"
+        else:
+            if self._source_id:
+                glib.source_remove(self._source_id)
+                
+            self._source_id = glib.timeout_add(5, self._timeout, target)
+
+    @threadslock
+    def _timeout(self, target):
+        current_value = int(self.pan.get_value() + 0.5)
+        new_value = current_value + cmp(target, current_value)
+        self.pan.set_value(new_value)
+
+        return new_value != target
+
+    @classmethod
+    def load_presets(cls, index):
+        for self in cls._instances:
+            self.load_preset(index)
+            
+    def set_values(self, value):
+        self.pan.set_value(value)
+        for each in self._presets:
+            each.set_value(value)
+            
+    def _cb_format_value(self, scale, value):
+        if value == 50:
+            return u"\u25C8"
+
+        pc = str(abs(int(value) * 2 - 100))
+        if value < 50:
+            return u"\u25C4 %s%%" % pc
+        
+        return u"%s%% \u25BA" % pc
+        
+
+class PanPresetButton(gtk.Button):
+    def __init__(self, labeltext):
+        self._labeltext = labeltext
+        gtk.Button.__init__(self)
+        self._label = gtk.Label(labeltext)
+        self.add(self._label)
+    
+    def highlight(self):
+        self._label.set_markup("<span foreground='red'>%s</span>" % self._labeltext)
+
+    def unhighlight(self):
+        self._label.set_text(self._labeltext)
+
+
+class PanPresetChooser(gtk.HBox):
+    def __init__(self):
+        gtk.HBox.__init__(self)
+        self.set_spacing(1)
+
+        label = gtk.Label(u"\u25C4")
+        self.pack_start(label)
+        label.show()
+
+        self.buttons = []
+        for i in range(PGlobs.num_panpresets):
+            button = PanPresetButton(str(i + 1))
+            self.pack_start(button, False)
+            button.show()
+            self.buttons.append(button)
+            button.connect_object("clicked", PanWidget.load_presets, i)
+            button.connect("clicked", self._cb_clicked)
+
+        label = gtk.Label(u"\u25BA")
+        self.pack_start(label)
+        label.show()
+            
+        set_tip(self, _('The pan preset selection buttons.\n\n'
+        'In the stereo image at a click the DJ can be on the left and a guest '
+        'on the right and when the guest is gone at a click the DJ can be '
+        'central again.\n\n'
+        'Note: preconfiguration of pan preset settings is required.'))
+
+    def load_preset(self, index):
+        try:
+            button = self.buttons[index]
+        except IndexError:
+            pass
+        else:
+            button.clicked()
+        
+    def _cb_clicked(self, clicked_button):
+        for button in self.buttons:
+            if button is clicked_button:
+                button.highlight()
+            else:
+                button.unhighlight()
+
 
 class AGCControl(gtk.Frame):
     mic_modes = (
@@ -361,52 +511,17 @@ class AGCControl(gtk.Frame):
         self.open.set_sensitive(False)
         self.fixups.append(lambda: self.open.emit("toggled"))
 
-        sizegroup = gtk.SizeGroup(gtk.SIZE_GROUP_HORIZONTAL)
-        panframe = gtk.Frame()
-        panframe.modes = (1, 2, 3)
-        set_tip(panframe, _('Stereo panning is the selection of where an audio '
-        'source sits from left to right within the stereo mix.\n\nThis control '
-        'maintains constant audio power throughout its range of travel, giving '
-        '-3dB attenuation in both audio channels at the half way point.\n\n'
-        'If you require 0dB straight down the middle then this feature should '
-        'be turned off.'))
-        
-        hbox = gtk.HBox()
-        self.pan_active = gtk.CheckButton(_('Stereo Panning'))
-        self.activedict[self.commandname + "_pan_active"] = self.pan_active
-        hbox.pack_start(self.pan_active, False, False, 0)
-        self.pan_active.show()
-        self.pan_active.connect("toggled", self.sendnewstats, "pan_active")
-        panframe.set_label_widget(hbox)
-        hbox.show()
-        self.fixups.append(lambda: self.pan_active.emit("toggled"))
-        
-        panvbox = gtk.VBox()
-        panvbox.set_border_width(1)
-        panframe.add(panvbox)
-        panhbox = gtk.HBox()
-        panvbox.pack_start(panhbox, False, False)
-        panhbox.set_spacing(3)
-        panhbox.set_border_width(3)
-        l = gtk.Label(_('L'))
-        sizegroup.add_widget(l)
-        panhbox.pack_start(l, False, False)
-        panadj = gtk.Adjustment(50.0, 0.0, 100.0, 1, 10)
-        self.pan = gtk.HScale(panadj)
-        self.pan.set_draw_value(False)
-        self.pan.connect("value-changed", self.sendnewstats, "pan")
-        self.pan.emit("value-changed")
-        self.fixups.append(lambda: self.pan.emit("value-changed"))
-        self.valuesdict[self.commandname + "_pan"] = self.pan
-        panhbox.pack_start(self.pan)
-        r = gtk.Label(_('R'))
-        sizegroup.add_widget(r)
-        panhbox.pack_start(r, False, False)
-        self.pan.add_mark(50.0, gtk.POS_BOTTOM, None)
-        self.pan.add_mark(25.0, gtk.POS_BOTTOM, None)
-        self.pan.add_mark(75.0, gtk.POS_BOTTOM, None)
-        self.vbox.pack_start(panframe, False, False)
-        panframe.show_all()
+        self.pan = PanWidget(_('Stereo Panning'), commandname)
+        self.pan.pan_active.connect("toggled", self.sendnewstats, "pan_active")
+        self.fixups.append(lambda: self.pan.pan_active.emit("toggled"))
+        self.pan.pan.connect("value-changed", self.sendnewstats, "pan")
+        self.pan.pan.emit("value-changed")
+        self.fixups.append(lambda: self.pan.pan.emit("value-changed"))
+        self.valuesdict.update(self.pan.valuesdict)
+        self.activedict.update(self.pan.activedict)
+
+        self.vbox.pack_start(self.pan, False, False)
+        self.pan.show_all()
 
         # TC: A set of controls that perform audio signal matching.
         pairedframe = gtk.Frame(" %s " % _('Signal Matching'))
@@ -619,8 +734,9 @@ class mixprefs:
                 for name, widget in self.valuesdict.iteritems():
                     f.write(name + "=" + str(widget.get_value()) + "\n")
                 for name, widget in self.textdict.iteritems():
-                    if widget.get_text() is not None:
-                        f.write(name + "=" + widget.get_text() + "\n")
+                    text = widget.get_text()
+                    if text is not None:
+                        f.write(name + "=" + text + "\n")
                     else:
                         f.write(name + "=\n")
         except IOError:
@@ -766,7 +882,7 @@ class mixprefs:
         self.notebook.set_current_page(self.notebook.page_num(self.aboutframe))
         self.window.present()
 
-    def fixup_mic_controls(self):
+    def mic_controls_backend_update(self):
         """Send mic preferences to the backend.
         
         This needs to be called whenever the backend is restarted.
@@ -774,6 +890,12 @@ class mixprefs:
         for mic in self.mic_controls:
             for fixup in mic.fixups:
                 fixup()
+                
+    def voip_pan_backend_update(self, widget=None):
+        widget = self.voip_pan
+        stringtosend = "VPAN=%d\nACTN=voippan\nend\n" % (widget.pan.get_value()
+                                    if widget.pan_active.get_active() else -1)
+        self.parent.mixer_write(stringtosend)
 
     def __init__(self, parent):
         self.parent = parent
@@ -1347,9 +1469,16 @@ class mixprefs:
             mic_controls[-1].set_partner(mic_controls[-2])  
         parent.mic_opener.finalise()
 
-        panevbox.pack_start(vbox, False, False, 0)
+        panevbox.pack_start(vbox, False)
         vbox.show()
                 
+        self.voip_pan = PanWidget(_('VoIP panning + mono downmix'), "voip_pan_widget")
+        self.voip_pan.pan_active.connect("toggled", self.voip_pan_backend_update)
+        self.voip_pan.pan.connect("value-changed", self.voip_pan_backend_update)
+        self.voip_pan.set_border_width(3)
+        panevbox.pack_start(self.voip_pan, False)
+        self.voip_pan.show_all()
+                
         label = gtk.Label(_('Channels'))
         self.notebook.append_page(scrolled_window, label)
         label.show()
@@ -1393,8 +1522,7 @@ class mixprefs:
         
         label = gtk.Label()
         label.set_markup(
-        '<span font_desc="sans 10">' +
-         _('Released under the GNU General Public License V2.0') + '</span>')
+        '<span font_desc="sans 10">' + PGlobs.license + '</span>')
         vbox.pack_start(label, False, False, 1)
         label.show()
         
@@ -1437,7 +1565,8 @@ class mixprefs:
             "Brian Millham (bmillham@users.sourceforge.net)"))
 
         contribs_page(_('Translators'),
-            ("<b>it</b>  Raffaele Morelli (raffaele.morelli@gmail.com)",))
+            ("<b>fr</b> nvignot (nicotux@users.sf.net)",
+            "<b>it</b>  Raffaele Morelli (raffaele.morelli@gmail.com)"))
 
         vbox.show()
 
@@ -1470,12 +1599,12 @@ class mixprefs:
             mic_controls[2].mode.set_active(1)
             mic_controls[2].alt_name.set_text("Aux L")
             mic_controls[2].groups_adj.set_value(2)
-            mic_controls[2].pan_active.set_active(True)
-            mic_controls[2].pan.set_value(0)
+            mic_controls[2].pan.pan_active.set_active(True)
+            mic_controls[2].pan.set_values(0)
             mic_controls[3].mode.set_active(3)
             mic_controls[3].alt_name.set_text("Aux R")
-            mic_controls[3].pan_active.set_active(True)
-            mic_controls[3].pan.set_value(100)
+            mic_controls[3].pan.pan_active.set_active(True)
+            mic_controls[3].pan.set_values(100)
             t = parent.mic_opener.ix2button[2].opener_tab
             t.button_text.set_text("Aux")
             t.icb.set_filename(FGlobs.pkgdatadir / "jack2.png")
@@ -1488,6 +1617,7 @@ class mixprefs:
 
         self.show_stream_meters.set_active(True)
         self.show_microphones.set_active(True)
+        self.show_background_tracks_player.set_active(True)
         self.show_button_bar.set_active(True)
 
         self.activedict = {  # Widgets to save that have the get_active method.
@@ -1511,6 +1641,8 @@ class mixprefs:
             "rg_adjust"     : self.rg_adjust,
             "str_meters"    : self.show_stream_meters,
             "mic_meters"    : self.show_microphones,
+            "btn_bar"       : self.show_button_bar,
+            "bg_tracks"     : self.show_background_tracks_player,
             "mic_meters_no_void" : self.no_mic_void_space,
             "players_visible"    : self.parent.menu.playersmenu_i
             }
@@ -1518,7 +1650,7 @@ class mixprefs:
         for each in itertools.chain(mic_controls, 
                             (self.parent.freewheel_button, self.songdbprefs,
                             self.lpconfig, self.rpconfig, opener_settings,
-                            label_subst)):
+                            label_subst, self.voip_pan)):
             self.activedict.update(each.activedict)
 
         self.valuesdict = {  # These widgets all have the get_value method.
@@ -1539,7 +1671,7 @@ class mixprefs:
             }
 
         for each in itertools.chain(mic_controls, (opener_settings,
-                                                            self.songdbprefs)):
+                                                            self.voip_pan)):
             self.valuesdict.update(each.valuesdict)
 
         self.textdict = {  # These widgets all have the get_text method.
